package de.hsel.spm.baudas.analysis;

import weka.core.Instance;
import weka.core.Instances;

import java.io.File;
import java.util.AbstractMap;
import java.util.HashMap;
import java.util.Map;

/**
 * count customers at all times.
 *
 * @author Johannes Theiner
 * @version 0.1
 * @since 1.0
 **/
public class ShoppingTimes implements Analysis<Map<Map.Entry<String, String>, Integer>> {

    private Instances instances;
    private Map<Map.Entry<String, String>, Integer> result;

    public ShoppingTimes(File file) {
        instances = load(file);
    }


    /**
     * get customer count at specific times.
     *
     * @return Map of Day-Hour Combinations and the corresponding user count
     */
    @Override
    public Map<Map.Entry<String, String>, Integer> getResult() {
        if (result != null) {
            return result;
        }

        result = new HashMap<>();

        for (Instance instance : instances) {
            Map.Entry<String, String> dayHour = new AbstractMap.SimpleImmutableEntry<>(instance.stringValue(Attribute.SHOPPING_DAY), instance.stringValue(Attribute.SHOPPING_HOUR));
            int tmp = 1;
            if (result.containsKey(dayHour))
                tmp = result.get(dayHour) + 1;

            result.put(dayHour, tmp);
        }
        return result;
    }
}