package de.hsel.spm.baudas.web.depiction;

import com.google.gson.Gson;
import de.hsel.spm.baudas.analysis.Cluster;
import de.hsel.spm.baudas.web.DatasetManagement;
import org.jetbrains.annotations.NotNull;

import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

/**
 * changes data from cluster analysis into a readable format.
 *
 * @author Julian Hinxlage
 * @version 0.1
 * @since 1.0
 **/
@WebServlet("/clusters")
public class ClusterTable extends HttpServlet implements Depiction {

    private static Map<UUID, Map> cache = new HashMap<>();

    /**
     * returns formatted analysis result to HTTP GET request from supplied id.
     *
     * @param req  request object
     * @param resp response object
     * @throws IOException failed to initialize print writer
     */
    @Override
    protected void doGet(@NotNull HttpServletRequest req, @NotNull HttpServletResponse resp) throws IOException {
        UUID uuid = init(req, resp, cache);
        if (uuid == null) return;


        File file = DatasetManagement.get(uuid);

        Cluster cluster = new Cluster(file);
        Map<Integer, Map<String, String>> result = cluster.getResult();

        PrintWriter out = resp.getWriter();
        Gson gson = new Gson();

        cache.put(uuid, result);
        out.print(gson.toJson(result));
    }
}
