package de.hsel.spm.baudas.web.depiction;

import com.google.gson.Gson;
import de.hsel.spm.baudas.analysis.SalesNumbers;
import de.hsel.spm.baudas.web.DatasetManagement;
import org.jetbrains.annotations.NotNull;

import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

/**
 * top/flop articles list diagram.
 *
 * @author Johannes Theiner
 * @version 0.1
 * @since 0.1
 **/

@WebServlet("/sales")
public class SalesNumberChart extends HttpServlet implements Depiction {

    private static Map<UUID, Map> cache = new HashMap<>();

    private static final long serialVersionUID = 6567531464214L;

    /**
     * returns formatted analysis result to HTTP GET request from supplied id.
     *
     * @param req  request object
     * @param resp response object
     * @throws IOException failed to initialize print writer
     */
    @Override
    protected void doGet(@NotNull HttpServletRequest req, @NotNull HttpServletResponse resp) throws IOException {
        UUID uuid = init(req, resp, cache);
        if (uuid == null) return;

        File file = DatasetManagement.get(uuid);

        SalesNumbers articles = new SalesNumbers(file);
        Map<String, Integer> map = articles.getResult();

        Map<String, List<String>> result = new HashMap<>();

        List<String> labels = new ArrayList<>();
        List<String> data = new ArrayList<>();

        for (Map.Entry<String, Integer> entry : map.entrySet()) {
            labels.add(entry.getKey());
            data.add(entry.getValue().toString());
        }

        result.put("labels", labels);
        result.put("data", data);

        PrintWriter out = resp.getWriter();
        Gson gson = new Gson();

        cache.put(uuid, result);
        out.print(gson.toJson(result));
    }
}