package de.hsel.spm.baudas.web.depiction;

import com.google.gson.Gson;
import de.hsel.spm.baudas.analysis.ShoppingCart;
import de.hsel.spm.baudas.web.DatasetManagement;
import org.jetbrains.annotations.NotNull;

import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.AbstractMap;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

/**
 * Changes data to readable format for table.
 *
 * @author Karsten Eden
 * @version 0.1
 * @since 0.1
 */
@WebServlet("/shopping_cart")
public class ShoppingCartTable extends HttpServlet implements Depiction {

    private static final long serialVersionUID = 5026732432605473505L;

    private static Map<UUID, Map> cache = new HashMap<>();

    /**
     * returns formatted analysis result to HTTP GET request from supplied id.
     *
     * @param req  request object
     * @param resp response object
     * @throws IOException failed to initialize print writer
     */
    @Override
    protected void doGet(@NotNull HttpServletRequest req, @NotNull HttpServletResponse resp) throws IOException {
        UUID uuid = init(req, resp, cache);
        if (uuid == null) return;


        List<Map.Entry<String, String>> tmp = new ArrayList<>();

        File file = DatasetManagement.get(uuid);

        ShoppingCart cart = new ShoppingCart(file);
        Map<List<String>, List<String>> map = cart.getResult();

        List<String> premises = new ArrayList<>();
        List<String> consequences = new ArrayList<>();

        for (Map.Entry<List<String>, List<String>> entry : map.entrySet()) {
            premises.add(String.join(", ", entry.getKey()));
            consequences.add(String.join(", ", entry.getValue()));
        }

        for (int i = 0; i < premises.size(); i++) {
            tmp.add(new AbstractMap.SimpleEntry<>(premises.get(i), consequences.get(i)));
        }
        Map<Integer, Map.Entry<String, String>> result = new HashMap<>();
        int i = 0;
        for (Map.Entry<String, String> entry : tmp) {
            result.put(i, entry);
            i++;
        }

        cache.put(uuid, result);

        PrintWriter out = resp.getWriter();
        Gson gson = new Gson();

        out.print(gson.toJson(result));
    }
}
