package de.hsel.spm.baudas.web.depiction;

import com.google.gson.Gson;
import de.hsel.spm.baudas.analysis.ShoppingTimes;
import de.hsel.spm.baudas.web.DatasetManagement;
import org.jetbrains.annotations.NotNull;

import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.AbstractMap;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import java.util.UUID;

/**
 * changes data from shopping times diagram into a readable format for chart.js
 *
 * @author Johannes Theiner
 * @version 0.1
 * @since 1.0
 **/
@WebServlet("/shopping_times")
public class ShoppingTimesChart extends HttpServlet implements Depiction {

    private static Map<UUID, Map> cache = new HashMap<>();

    private static final long serialVersionUID = 6567531484L;

    /**
     * returns formatted analysis result to HTTP GET request from supplied id.
     *
     * @param req  request object
     * @param resp response object
     * @throws IOException failed to initialize print writer
     */
    @Override
    protected void doGet(@NotNull HttpServletRequest req, @NotNull HttpServletResponse resp) throws IOException {
        UUID uuid = init(req, resp, cache);
        if (uuid == null) return;


        List<String> definedOrder = Arrays.asList("<10 Uhr", "10-12 Uhr", "12-14 Uhr", "14-17 Uhr", ">17 Uhr");
        Comparator<String> comparator = Comparator.comparingInt(definedOrder::indexOf);

        Map<String, Collection<String>> result = new HashMap<>();

        File file = DatasetManagement.get(uuid);

        ShoppingTimes shoppingTimes = new ShoppingTimes(file);
        Map<Map.Entry<String, String>, Integer> map = shoppingTimes.getResult();

        Set<String> days = new TreeSet<>();
        Set<String> hours = new TreeSet<>(comparator);
        for (Map.Entry<String, String> dayHour : map.keySet()) {
            days.add(dayHour.getKey());
            hours.add(dayHour.getValue());
        }
        result.put("labels", hours);


        for (String day : days) {
            if (!result.containsKey(day)) {
                result.put(day, new ArrayList<>());
            }
            for (String hour : hours) {
                if (map.containsKey(new AbstractMap.SimpleEntry<>(day, hour))) {
                    result.get(day).add(map.get(new AbstractMap.SimpleEntry<>(day, hour)).toString());
                } else result.get(day).add("0");
            }
        }
        PrintWriter out = resp.getWriter();
        Gson gson = new Gson();

        cache.put(uuid, result);
        out.print(gson.toJson(result));

    }
}