package de.hsel.spm.baudas.analysis;

import org.junit.jupiter.api.Test;

import java.io.File;
import java.net.URL;
import java.time.Duration;
import java.util.Map;
import java.util.concurrent.atomic.AtomicReference;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTimeout;


/**
 * Week Overview Analysis Test.
 *
 * @author Johannes Theiner
 * @version 0.1
 * @since 0.1
 **/
class WeekOverviewTest {

    @Test
    void test100() {
        URL url = getClass().getClassLoader().getResource("kd100.csv");
        assert url != null;

        WeekOverview overview = new WeekOverview(new File(url.getFile()));

        AtomicReference<Map<String, Map.Entry<Double, Integer>>> result = new AtomicReference<>();

        assertTimeout(Duration.ofMillis(15), () -> result.set(overview.getResult()));

        assertEquals(2477, result.get().get("Montag").getKey());
        assertEquals(3359, result.get().get("Dienstag").getKey());
        assertEquals(1436, result.get().get("Mittwoch").getKey());
        assertEquals(2472, result.get().get("Donnerstag").getKey());
        assertEquals(2399, result.get().get("Freitag").getKey());
        assertEquals(18273, result.get().get("Samstag").getKey());

        assertEquals(8, result.get().get("Montag").getValue());
        assertEquals(13, result.get().get("Dienstag").getValue());
        assertEquals(5, result.get().get("Mittwoch").getValue());
        assertEquals(9, result.get().get("Donnerstag").getValue());
        assertEquals(10, result.get().get("Freitag").getValue());
        assertEquals(55, result.get().get("Samstag").getValue());
    }

    @Test
    void test1000() {
        URL url = getClass().getClassLoader().getResource("kd1000.csv");
        assert url != null;

        WeekOverview overview = new WeekOverview(new File(url.getFile()));

        AtomicReference<Map<String, Map.Entry<Double, Integer>>> result = new AtomicReference<>();

        assertTimeout(Duration.ofMillis(15), () -> result.set(overview.getResult()));

        assertEquals(26273, result.get().get("Montag").getKey());
        assertEquals(23345, result.get().get("Dienstag").getKey());
        assertEquals(25203, result.get().get("Mittwoch").getKey());
        assertEquals(26332, result.get().get("Donnerstag").getKey());
        assertEquals(26413, result.get().get("Freitag").getKey());
        assertEquals(146279, result.get().get("Samstag").getKey());

        assertEquals(108, result.get().get("Montag").getValue());
        assertEquals(93, result.get().get("Dienstag").getValue());
        assertEquals(89, result.get().get("Mittwoch").getValue());
        assertEquals(90, result.get().get("Donnerstag").getValue());
        assertEquals(110, result.get().get("Freitag").getValue());
        assertEquals(510, result.get().get("Samstag").getValue());
    }

    @Test
    void test10000() {
        URL url = getClass().getClassLoader().getResource("kd10000.csv");
        assert url != null;

        WeekOverview overview = new WeekOverview(new File(url.getFile()));

        AtomicReference<Map<String, Map.Entry<Double, Integer>>> result = new AtomicReference<>();

        assertTimeout(Duration.ofMillis(50), () -> result.set(overview.getResult()));

        assertEquals(295688, result.get().get("Montag").getKey());
        assertEquals(267269, result.get().get("Dienstag").getKey());
        assertEquals(269610, result.get().get("Mittwoch").getKey());
        assertEquals(254510, result.get().get("Donnerstag").getKey());
        assertEquals(330907, result.get().get("Freitag").getKey());
        assertEquals(1410789, result.get().get("Samstag").getKey());

        assertEquals(1058, result.get().get("Montag").getValue());
        assertEquals(942, result.get().get("Dienstag").getValue());
        assertEquals(915, result.get().get("Mittwoch").getValue());
        assertEquals(919, result.get().get("Donnerstag").getValue());
        assertEquals(1177, result.get().get("Freitag").getValue());
        assertEquals(4989, result.get().get("Samstag").getValue());
    }
}
